@@ -27,6 +27,13 @@
 #   Instead set set_x_mrg and set_y_mrg
 # o Changed default X and Y Margin to 1.
 # o Added wxCalendar.SetMargin.
+#
+# 17/03/2004 - Joerg "Adi" Sieker adi@sieker.info
+# o Added keyboard navigation to the control.
+#   Use the cursor keys to navigate through the ages. :)
+#   The Home key function as go to today
+# o select day is now a filled rect instead of just an outline
+
 
 from wxPython.wx import *
 
@@ -82,8 +89,8 @@ def DefaultColors():
     colors[COLOR_FONT] = wxSystemSettings_GetColour(wxSYS_COLOUR_WINDOWTEXT)
     colors[COLOR_3D_LIGHT] = wxSystemSettings_GetColour(wxSYS_COLOUR_BTNHIGHLIGHT)
     colors[COLOR_3D_DARK] = wxSystemSettings_GetColour(wxSYS_COLOUR_BTNSHADOW)
-    colors[COLOR_HIGHLIGHT_FONT]  =  'PINK'
-    colors[COLOR_HIGHLIGHT_BACKGROUND] = 'RED'
+    colors[COLOR_HIGHLIGHT_FONT]  =  wxSystemSettings_GetColour(wxSYS_COLOUR_HIGHLIGHTTEXT)
+    colors[COLOR_HIGHLIGHT_BACKGROUND] = wxSystemSettings_GetColour(wxSYS_COLOUR_HIGHLIGHT)
     return colors
 
 # calendar drawing routing
@@ -202,14 +209,26 @@ class CalDraw:
         for val in list:
             self.cal_sel[val] = (cfont, cbackgrd)
 
-    def DrawBorder(self, DC):   # draw border around the outside of the main display rectangle
-        brush = wxBrush(MakeColor(self.colors[COLOR_BACKGROUND]), wxSOLID)
-        DC.SetBrush(brush)
-        DC.SetPen(wxPen(MakeColor(self.colors[COLOR_BORDER]), 1))
-        
+    # draw border around the outside of the main display rectangle
+    def DrawBorder(self, DC, transparent = False):   
         if self.outer_border is True:
-            rect = wxRect(self.cx_st, self.cy_st, self.sizew, self.sizeh)  # full display window area
-            DC.DrawRectangle(rect.x, rect.y, rect.width, rect.height)
+            if transparent == False:
+                brush = wxBrush(MakeColor(self.colors[COLOR_BACKGROUND]), wxSOLID)
+            else:
+                brush = wxTRANSPARENT_BRUSH
+            DC.SetBrush(brush)
+            DC.SetPen(wxPen(MakeColor(self.colors[COLOR_BORDER])))
+            # full display window area
+            rect = wxRect(self.cx_st, self.cy_st, self.sizew, self.sizeh)  
+            DC.DrawRectangleRect(rect)
+
+    def DrawFocusIndicator(self, DC):
+        if self.outer_border is True:
+            DC.SetBrush(wxTRANSPARENT_BRUSH)
+            DC.SetPen(wxPen(MakeColor(self.colors[COLOR_HIGHLIGHT_BACKGROUND]), style=wxDOT))
+            # full display window area
+            rect = wxRect(self.cx_st, self.cy_st, self.sizew, self.sizeh)  
+            DC.DrawRectangleRect(rect)
 
     def DrawNumVal(self):
         self.DrawNum()
@@ -263,6 +282,9 @@ class CalDraw:
 
     def GetRect(self):      # get the display rectange list of the day grid
         cnt = 0
+        h = 0
+        w = 0
+        
         for y in self.gridy[1:-1]:
             if y == self.gridy[-2]:
                 h = h + self.restH
@@ -411,9 +433,7 @@ class CalDraw:
             DC.DrawText(day, x+diffx, y+diffy)
             cnt_x = cnt_x + 1
 
-    def DrawNum(self, DC):      # draw the day numbers
-        f = wxFont(10, self.font, wxNORMAL, self.bold)      # initial font setting
-
+    def _CalcFontSize(self, DC, f):
         if self.num_auto == True:
             test_size = self.max_num_size      # max size
             test_day = ' 99 '
@@ -431,13 +451,18 @@ class CalDraw:
             f.SetPointSize(self.num_size)   # set fixed size
             DC.SetFont(f)
 
+    # draw the day numbers
+    def DrawNum(self, DC):      
+        f = wxFont(10, self.font, wxNORMAL, self.bold)      # initial font setting
+        self._CalcFontSize(DC, f)
+
         cnt_x = 0
         cnt_y = 1
         for val in self.cal_days:
             x = self.gridx[cnt_x]
             y = self.gridy[cnt_y]
 
-            self.DrawDayText(x, y, val, f, DC)
+            self._DrawDayText(x, y, val, f, DC)
 
             if cnt_x < 6:
                 cnt_x = cnt_x + 1
@@ -445,7 +470,7 @@ class CalDraw:
                 cnt_x = 0
                 cnt_y = cnt_y + 1
 
-    def DrawDayText(self, x, y, text, font, DC):
+    def _DrawDayText(self, x, y, text, font, DC):
         try:
             num_val = int(text)
             num_color = self.cal_sel[num_val][0]
@@ -456,7 +481,7 @@ class CalDraw:
         DC.SetFont(font)
 
         tw,th = DC.GetTextExtent(text)
-
+        
         if self.num_align_horz == wxALIGN_CENTRE:
             adj_h = (self.cellW - tw)/2
         elif self.num_align_horz == wxALIGN_RIGHT:
@@ -477,6 +502,17 @@ class CalDraw:
 
         DC.DrawText(text, x+adj_h, y+adj_v)
 
+    def DrawDayText(self, DC, key):
+        f = wxFont(10, self.font, wxNORMAL, self.bold)      # initial font setting
+        self._CalcFontSize(DC, f)
+
+        val = self.cal_days[key]
+        cnt_x = key % 7
+        cnt_y = int(key / 7)+1
+        x = self.gridx[cnt_x]
+        y = self.gridy[cnt_y]
+        self._DrawDayText(x, y, val, f, DC)
+
     def Center(self):       # calculate the dimensions in the center of the drawing area
         borderW = self.x_mrg * 2
         borderH = self.y_mrg + self.y_end + self.title_offset
@@ -486,7 +522,8 @@ class CalDraw:
         
         self.restW = ((self.sizew - borderW)%7 ) - 1
         
-        self.weekHdrCellH = int(self.cellH * self.cal_week_scale)     # week title adjustment
+        # week title adjustment
+        self.weekHdrCellH = int(self.cellH * self.cal_week_scale)
         # recalculate the cell height exkl. the week header and
         # subtracting the size
         self.cellH = int((self.sizeh - borderH - self.weekHdrCellH)/6)
@@ -496,6 +533,7 @@ class CalDraw:
         self.calH = self.cellH * 6 + self.weekHdrCellH
 
     def DrawSel(self, DC):         # highlighted selected days
+        
         for key in self.cal_sel.keys():
             sel_color = self.cal_sel[key][1]
             brush = wxBrush(MakeColor(sel_color), wxSOLID)
@@ -578,10 +616,13 @@ class PrtCalDraw(CalDraw):
     def SetPreview(self, preview):
         self.preview = preview
 
-class wxCalendar(wxWindow):
-    def __init__(self, parent, id, pos=wxDefaultPosition, size=wxDefaultSize):
-        wxWindow.__init__(self, parent, id, pos, size)
+class wxCalendar( wxPyControl ):
+    def __init__(self, parent, id, pos=wxDefaultPosition, size=(400,400),
+                   style= 0, validator=wxDefaultValidator,
+                   name= "calendar"):
+        wxPyControl.__init__(self, parent, id, pos, size, style | wxWANTS_CHARS, validator, name)
 
+        self.hasFocus = False
         # set the calendar control attributes
         self.cal = None
         
@@ -604,6 +645,9 @@ class wxCalendar(wxWindow):
         self.Connect(-1, -1, wxEVT_LEFT_DCLICK, self.OnLeftDEvent)
         self.Connect(-1, -1, wxEVT_RIGHT_DOWN, self.OnRightEvent)
         self.Connect(-1, -1, wxEVT_RIGHT_DCLICK, self.OnRightDEvent)
+        self.Connect(-1, -1, wxEVT_SET_FOCUS, self.OnSetFocus)
+        self.Connect(-1, -1, wxEVT_KILL_FOCUS, self.OnKillFocus)
+        self.Connect(-1, -1, wxEVT_KEY_DOWN, self.OnKeyDown)
 
         self.sel_key = None      #  last used by
         self.sel_lst = []        # highlighted selected days
@@ -616,6 +660,9 @@ class wxCalendar(wxWindow):
         EVT_PAINT(self, self.OnPaint)
         EVT_SIZE(self, self.OnSize)
 
+    def AcceptsFocus(self):
+        return self.IsShown() and self.IsEnabled()
+
     def GetColor(self, name):
         return MakeColor(self.colors[name])
 
@@ -657,6 +704,65 @@ class wxCalendar(wxWindow):
         self.click = 'DRIGHT'
         self.ProcessClick(event)
 
+    def OnSetFocus(self, event):
+        self.hasFocus = True
+        self.DrawFocusIndicator(True)
+
+    def OnKillFocus(self, event):
+        self.hasFocus = False
+        self.DrawFocusIndicator(False)
+
+    def OnKeyDown(self, event):
+        if not self.hasFocus:
+            event.Skip()
+            return
+        
+        key_code = event.KeyCode()
+        
+        if key_code == WXK_TAB:
+            forward = not event.ShiftDown()
+            ne = wxNavigationKeyEvent()
+            ne.SetDirection(forward)
+            ne.SetCurrentFocus(self)
+            ne.SetEventObject(self)
+            self.GetParent().GetEventHandler().ProcessEvent(ne)
+            event.Skip()
+            return
+
+        delta = None
+
+        if key_code == WXK_UP:
+            delta = -7 
+        elif key_code == WXK_DOWN:
+            delta = 7 
+        elif key_code == WXK_LEFT:
+            delta = -1 
+        elif key_code == WXK_RIGHT:
+            delta = 1 
+        elif key_code == wx.WXK_HOME:
+            curDate = wxDateTimeFromDMY(int(self.cal_days[self.sel_key]),self.month - 1,self.year)
+            newDate = wxDateTime_Now()
+            ts = newDate - curDate
+            delta =  ts.GetDays()
+
+        if delta <> None:
+            curDate = wxDateTimeFromDMY(int(self.cal_days[self.sel_key]),self.month - 1,self.year)
+            timeSpan = wxTimeSpan_Days(delta)
+            newDate = curDate + timeSpan
+
+            if curDate.GetMonth() == newDate.GetMonth():
+                self.set_day = newDate.GetDay()
+                key = self.sel_key + delta
+                self.SelectDay(key)
+            else:
+                self.month = newDate.GetMonth() + 1
+                self.year = newDate.GetYear()
+                self.set_day = newDate.GetDay()
+                self.sel_key = None
+                self.DoDrawing(wxClientDC(self))
+
+        event.Skip()
+
     def SetSize(self, set_size):
         self.size = set_size
         
@@ -840,7 +946,36 @@ class wxCalendar(wxWindow):
         DC.EndDrawing()
 
     # draw the selection rectangle
-    def DrawRect(self, key, fgcolor = None, width = 0):
+    def DrawFocusIndicator(self, draw):
+        DC = wxClientDC(self)
+        try:
+            if draw == True:
+                self.caldraw.DrawFocusIndicator(DC)
+            else:
+                self.caldraw.DrawBorder(DC,True)
+        except:
+            pass
+
+    def DrawRect(self, key, bgcolor = 'WHITE', fgcolor= 'PINK',width = 0):
+        if key == None:
+            return
+
+        DC = wxClientDC(self)
+        DC.BeginDrawing()
+
+        brush = wxBrush(MakeColor(bgcolor))
+        DC.SetBrush(brush)
+
+        DC.SetPen(wxTRANSPARENT_PEN)
+
+        rect = self.rg[key]
+        DC.DrawRectangle(rect.x+1, rect.y+1, rect.width-2, rect.height-2)
+
+        self.caldraw.DrawDayText(DC,key)
+
+        DC.EndDrawing()
+        
+    def DrawRectOrg(self, key, fgcolor = 'BLACK', width = 0):
 
         if key == None:
             return
@@ -867,12 +1002,24 @@ class wxCalendar(wxWindow):
         day = day + self.st_pos - 1
         self.SelectDay(day)
 
+    def IsDayInWeekend(self, key):
+        try:
+            t = Date(self.year, self.month, 1)
+            
+            day = self.cal_days[key]
+            day = int(day) + t.day_of_week
+            
+            if day % 7 == 6 or day % 7 == 0:
+                return True
+        except:
+            return False
+        
     def SelectDay(self, key):
         sel_size = 1
-        self.DrawRect(self.sel_key, self.GetColor(COLOR_BACKGROUND), sel_size)     # clear large selection
-
-        if self.hide_grid is False:
-            self.DrawRect(self.sel_key, self.GetColor(COLOR_GRID_LINES),sel_size )
+        
+        if self.sel_key != None:
+            (cfont, bgcolor) = self.__GetColorsForDay(self.sel_key)
+            self.DrawRect(self.sel_key, bgcolor,cfont, sel_size)
 
         self.DrawRect(key, self.GetColor(COLOR_HIGHLIGHT_BACKGROUND), sel_size)
         self.sel_key = key      # store last used by
@@ -884,6 +1031,21 @@ class wxCalendar(wxWindow):
         self.set_x_mrg = xmarg
         self.set_y_mrg = ymarg
         self.set_y_end = ymarg
+    def __GetColorsForDay(self, key):
+        cfont   = self.GetColor(COLOR_FONT)
+        bgcolor = self.GetColor(COLOR_BACKGROUND)
+
+        if self.IsDayInWeekend(key) is True and self.show_weekend is True:
+            cfont   = self.GetColor(COLOR_WEEKEND_FONT)
+            bgcolor = self.GetColor(COLOR_WEEKEND_BACKGROUND)
+
+        try:
+            dayIdx = int(self.cal_days[key])
+            (cfont, bgcolor) = self.caldraw.cal_sel[dayIdx]
+        except:
+            pass
+
+        return (cfont, bgcolor)
 
 class CalenDlg(wxDialog):
     def __init__(self, parent, month=None, day = None, year=None):
